function nn1()
h = 12; % number of neurons in hidden layer
ns = 100; % num samples (per class)
dim = 2;
nc = 3; % num classes
N = ns*nc; % total number of samples
reg = 1e-3; % (0.001) Weight regularization factor
step_size = 1e-0;
[X y] = datagen(ns, dim, nc);,
% X is ns * dim
% y is ns * 1 (the class of each sample in X)

% First hidden layer with RELU activation
W1 = randn(dim, h);
b1 = zeros(1,h);
% Output layer with softmax-cross entropy output
W2 = randn(h, nc);
b2 = zeros(1,nc);
%vidObj = VideoWriter('nn_optimization.avi');
%open(vidObj);
fh = figure;
% plot data loss
loss_arr = [];
% Y1: output of first hidden layer before applying activation
% A1: output after applying activation
for numIter = 1: 2000,
    %%%%%%%%%% Forward pass %%%%%%%%%%
    Y1 = X*W1 + ones(N,1)*b1;
    % apply RELU non-linearity
    A1 = max(0, Y1); % size: N*H
    % compute scores. % scores is ns * nc (tells us the class scores for each data sample) 
    scores = A1*W2 + ones(N,1)*b2;
    % get un-normalized probabilities
    exp_scores = exp(scores);
    
    % normalize for each sample (each row sums to 1), so each score in a row is
    % a probability
    probs = exp_scores./sum(exp_scores, 2);
    
    % -log(probability of the correct class)
    data_loss = 0;
    for i = 1:N
        % if prob at index y(i) is 1 (forward pass assigned max probability
        % to correct class), loss is 0, otherwise it is non-zero
        data_loss = data_loss + -1*log(probs(i, y(i)));
    end
    % loss
    data_loss = data_loss/N;
    reg_loss = 0.5*reg*sum(sum(W1.*W1)) + 0.5*reg*sum(sum(W2.*W2));
    loss = data_loss + reg_loss;
    loss_arr(end+1) = loss;
    
    % For making the movie showing decision boundaries
    if mod(numIter,10) == 0
        %    visualize_decision_boundaries(X, y, W1, b1, W2, b2, fh, vidObj);
    end
    
    %%%%%%%%%% Backward pass %%%%%%%%%%
    
    % derivative of the loss wrt scores (ns * nc)
    dloss_dscores = probs;
    % could probably do a vector notation here, but use a loop for now
    for i = 1:N
        dloss_dscores(i, y(i)) = dloss_dscores(i, y(i)) - 1;
    end
    dloss_dscores = dloss_dscores/N; % size:N*nc
    dscores_dW2 = A1'; % H*N
    dloss_dW2 = dscores_dW2*dloss_dscores; %H*nc
    dloss_dW2 = dloss_dW2 + reg*W2;
    dscores_db2 = ones(1, N);
    dloss_db2 = dscores_db2*dloss_dscores;
    
    % propagate gradient back through the weight layer
    dhidden = dloss_dscores*W2'; % (N*nc)*(H*nc)' = N*H
    % Now propagate back through RELU layer
    dhidden(Y1 <= 0) = 0;
    
    % calculate corrections for W1
    dloss_dW1 = X'*dhidden + reg*W1; %(N*dim)'*(N*H)
    dloss_db1 = ones(1,N)*dhidden;
    
    % update weights and bias. There are many strategies available to
    % adjust the step_size as the optimization proceeds towards the minimum
    % and the error surface gets flatter. In this simple example, we use a
    % constant step_size
    W2 = W2 - dloss_dW2*step_size;
    b2 = b2 - dloss_db2*step_size;
    
    W1 = W1 - dloss_dW1*step_size;
    b1 = b1 - dloss_db1*step_size;
end
%close(vidObj);
% evaluate training accuracy
% find indicies where the max probability for each class occurs
[val idx] = max(probs, [], 2);
% correct classification is when idx matches
num_correct_classifications = sum(idx == y);
% classification accuracy
acc = num_correct_classifications/N;
